package com.perforce.maven.scm.provider.p4;

/*
 * Copyright (c) 2010 Perforce Software. All rights reserved.
 */

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import org.apache.maven.settings.Server;
import org.apache.maven.settings.Settings;
import org.apache.maven.settings.io.xpp3.SettingsXpp3Reader;
import org.codehaus.plexus.util.FileUtils;
import org.codehaus.plexus.util.ReaderFactory;
import org.codehaus.plexus.util.cli.CommandLineException;
import org.codehaus.plexus.util.cli.CommandLineUtils;
import org.codehaus.plexus.util.cli.Commandline;
import org.codehaus.plexus.util.cli.StreamConsumer;
import org.codehaus.plexus.util.xml.pull.XmlPullParserException;

import com.perforce.maven.scm.provider.p4.util.P4Utils;

public final class P4ScmTestUtils
{
    private static String scmTckUrl = null;

    private P4ScmTestUtils()
    {
    }

    public static void initRepo()
        throws IOException
    {
        // during tck test we dont want p4maven-settings.xml get involved, we have a separate test for it
        P4Utils.setP4MavenSettingsDirecotry( new File( "some-where-very-bogus" ) );

        if ( scmTckUrl != null )
        {
            return;
        }

        scmTckUrl = System.getProperty( "scmTckUrl" );

        if ( scmTckUrl == null )
        {
            // try the default one with user/password from settings.xml
            Settings settings = readMavenSettings();
            Server server = settings.getServer( "localhost:1666" );
            if ( server != null )
            {
                StringBuffer buff = new StringBuffer();
                buff.append( "scm:p4:" ).append( server.getUsername() ).append( ":" ).append( server.getPassword() ).append( "@" ).append( "localhost:1666://scmtck/p4maven-tck" );
                scmTckUrl = buff.toString();
            }

            if ( scmTckUrl == null )
            {
                throw new IllegalArgumentException( "Please configure system property for scmTckUrl" );

            }
        }

    }

    public static void populateOriginalData( String baseDir, File workingCopy )
        throws IOException
    {
        FileUtils.copyDirectoryStructure( new File( baseDir, "src/it/initial-data/src/data/p4maven-tck" ), workingCopy );
    }

    public static String getScmUrl()
    {
        return scmTckUrl;
    }

    public static void executeCommandLine( Commandline cl, StreamConsumer stdout, StreamConsumer stderr )
        throws CommandLineException
    {
        if ( CommandLineUtils.executeCommandLine( cl, stdout, stderr ) != 0 )
        {
            throw new CommandLineException( cl.toString() + " exits with none zero code." );
        }
    }

    private static final String[] ADDED_FILES =
        { "project.xml", "src/main/java/Foo.java", "src/main/java/org/Foo.java" };

    public static String getAddedIncludes()
    {
        StringBuffer buffer = new StringBuffer();
        for ( String file : ADDED_FILES )
        {
            buffer.append( file );
            if ( !file.equals( ADDED_FILES[ADDED_FILES.length - 1] ) )
            {
                buffer.append( "," );
            }
        }

        return buffer.toString();
    }

    private static final String[] TEST_FILES = { "pom.xml", "readme.txt", "src/main/java/Application.java",
        "src/test/java/Test.java" };

    public static String getTestIncludes()
    {
        StringBuffer buffer = new StringBuffer();
        for ( String file : TEST_FILES )
        {
            buffer.append( file );
            if ( !file.equals( TEST_FILES[TEST_FILES.length - 1] ) )
            {
                buffer.append( "," );
            }
        }

        return buffer.toString();
    }

    private static Settings readMavenSettings()
    {
        File settingsFile = new File( System.getProperty( "user.home" ), ".m2/settings.xml" );

        Settings settings = new Settings();

        if ( settingsFile.exists() )
        {
            SettingsXpp3Reader reader = new SettingsXpp3Reader();
            try
            {
                settings = reader.read( ReaderFactory.newXmlReader( settingsFile ) );
            }
            catch ( FileNotFoundException e )
            {
                // nop
            }
            catch ( IOException e )
            {
                // nop
            }
            catch ( XmlPullParserException e )
            {
                String message = settingsFile.getAbsolutePath() + " isn't well formed. SKIPPED." + e.getMessage();

                System.err.println( message );
            }
        }

        return settings;
    }
}
